; sub routine calls
;TICK:												; for test purpose
;		in		Temp, PORTB
;		ldi		Temp2, 1<<0
;		eor		Temp, Temp2
;		out		PORTB, Temp
;		ret

SUBROUTINES:
		rcall	CHECK_BUTTONS							; check if a button is pressed
		rcall	CALC_PWM								; calculate and output the duty ratio
		andi	Status, ~(1<<ACT_BIT)					; reset the activation bit of the sub routine calls
		ret

CHECK_BUTTONS:											; check the buttons states
		mov		Temp, Status							; check the ports and set the State variable
		andi	Temp, ~((1<<TUPIN)|(1<<TDOWNIN)|(1<<TMODEIN))
		sbis	PINTUP, TUP
		ori		Temp, 1<<TUPIN
		sbis	PINTDOWN, TDOWN
		ori		Temp, 1<<TDOWNIN
		sbis	PINTMOD, TMOD
		ori		Temp, 1<<TMODEIN
		mov		Temp2, Temp								; make a backup
		eor		Temp2, Status
		breq	BUTTONS_STATE_EQUAL						; detoggling ok
		mov		Status, Temp							; detogglinh not ok, check a next time
		rjmp	CHECK_BUTTONS_OUT
 BUTTONS_STATE_EQUAL:
		sbrc	OLDSTATUS, TUPIN						; check if one of the buttons is pressed for longer time
		rjmp	CHK_BUT_OLDSTATE_PRESSED
		sbrc	OLDSTATUS, TDOWNIN
		rjmp	CHK_BUT_OLDSTATE_PRESSED
		sbrc	OLDSTATUS, TMODEIN
		rjmp	CHK_BUT_OLDSTATE_PRESSED		
		sbrc	Status, TUPIN							; no previously pressed button, test if the state should be changed
		rjmp	BUTTONS_TUPIN_PRESSED
		sbrc	Status, TDOWNIN
		rjmp	BUTTONS_TDOWNIN_PRESSED
		sbrc	Status, TMODEIN
		rjmp	BUTTONS_TMODEIN_PRESSED
		rjmp	CHECK_BUTTONS_OUT

 BUTTONS_TUPIN_PRESSED:									; TUP is pressed
		sbrc	Status, HIGHMODE
		rjmp	B_TU_P_INC_HIGH_DUTY
		sbrc	Status, PROGMODE
		rjmp	B_TU_P_INC_MAX_DUTY						; in switching mode, increase the maximum duty ratio
		ldi		Temp2, LinCoeff
		rcall	EEP_READ
		cpi		Temp, 63								; test if max. value already achieved
		brsh	CHECK_BUTTONS_OUT
		inc		Temp
		rcall	EEP_ATOMIC_WRITE						; write the new value
		rjmp	CHECK_BUTTONS_OUT
  B_TU_P_INC_MAX_DUTY:									; in switching mode, increase the max. duty ratio
		ldi		Temp2, MaxDuty
		rcall	EEP_READ								; read the max. duty ratio
		cpi		Temp, 255								; test if max. value already achieved
		brsh	CHECK_BUTTONS_OUT
		inc		Temp
		rcall	EEP_ATOMIC_WRITE
		rjmp	CHECK_BUTTONS_OUT
  B_TU_P_INC_HIGH_DUTY:									; in switching mode, increase the max. duty ratio
		ldi		Temp2, HighDuty
		rcall	EEP_READ								; read the max. duty ratio
		cpi		Temp, 255								; test if max. value already achieved
		brsh	CHECK_BUTTONS_OUT
		inc		Temp
		rcall	EEP_ATOMIC_WRITE
		rjmp	CHECK_BUTTONS_OUT
 BUTTONS_TDOWNIN_PRESSED:								; TDOWN is pressed
		sbrc	Status, HIGHMODE
		rjmp	B_TD_DEC_HIGH_DUTY
		sbrc	Status, PROGMODE
		rjmp	B_TD_DEC_MAX_DUTY						; in switching mode, decrease the maximum duty ratio
		ldi		Temp2, LinCoeff							; in linear mode, decrease the linear coeffifient
		rcall	EEP_READ
		cpi		Temp, 2									; test if already at minimum
		brlo	CHECK_BUTTONS_OUT
		dec		Temp
		rcall	EEP_ATOMIC_WRITE
		rjmp	CHECK_BUTTONS_OUT
  B_TD_DEC_MAX_DUTY:									; in switching mode, decrease the maximum duty ratio
		ldi		Temp2, MaxDuty							; decrease the max. duty ratio
		rcall	EEP_READ
		cpi		Temp, PWMMin							; test if already at minimum
		brlo	CHECK_BUTTONS_OUT
		dec		Temp									; decrease the duty ratio
		rcall	EEP_ATOMIC_WRITE
		rjmp	CHECK_BUTTONS_OUT
  B_TD_DEC_HIGH_DUTY:									; in switching mode, decrease the high duty ratio
		ldi		Temp2, HighDuty							; decrease the high. duty ratio
		rcall	EEP_READ
		cpi		Temp, PWMMin							; test if already at minimum
		brlo	CHECK_BUTTONS_OUT
		dec		Temp									; decrease the duty ratio
		rcall	EEP_ATOMIC_WRITE
		rjmp	CHECK_BUTTONS_OUT
 BUTTONS_TMODEIN_PRESSED:								; TMODE is pressed, change the operation mode
		ori		OLDSTATUS, (1<<TMODEIN)
		sbrc	Status, HIGHMODE
		rjmp	B_T_P_HIGH
		sbrc	Status, PROGMODE
		rjmp	B_T_P_MAX
  B_T_P_LIN:											; linear mode, change to max. mode
		ori		Status, 1<<PROGMODE
		rjmp	CHECK_BUTTONS_OUT
  B_T_P_MAX:											; max mode, change to high mode
		ori		Status, 1<<HIGHMODE
		rjmp	CHECK_BUTTONS_OUT
  B_T_P_HIGH:											; high mode, change to linear mode
		andi		Status, ~((1<<PROGMODE)|(1<<HIGHMODE))
		rjmp	CHECK_BUTTONS_OUT
 CHK_BUT_OLDSTATE_PRESSED:								; at least one button is pressed, wait until released
		sbrc	Temp, TUPIN
		rjmp	CHECK_BUTTONS_OUT
		andi	OLDSTATUS, ~(1<<TUPIN)
		sbrc	Temp, TDOWNIN
		rjmp	CHECK_BUTTONS_OUT
		andi	OLDSTATUS, ~(1<<TDOWNIN)
		sbrc	Temp, TMODEIN
		rjmp	CHECK_BUTTONS_OUT
		andi	OLDSTATUS, ~(1<<TMODEIN)
		rjmp	CHECK_BUTTONS_OUT
 CHECK_BUTTONS_OUT:
		ret

CALC_PWM:												; calculate the PWM duty ratio from the measured ADC value
		;cpi		StartCounter, TimeStart+1
		;brlo	CALC_PWM_STARTCNT_OK
		;ldi		StartCounter, TimeStart
 CALC_PWM_STARTCNT_OK:
		sbrc	Status, ONSTATE
		rjmp	CALC_PWM_MOT_ON
 CALC_PWM_MOT_OFF:
		cpi		StartCounter, TimeStart
		brsh	CALC_PWM_MOT_OFF_LONG
		inc		StartCounter
 CALC_PWM_MOT_OFF_LONG:
 		lds		Temp2, AnaResH
		lds		Temp, AnaResL
		subi	Temp, LOW(ValStart)
		sbci	Temp2, HIGH(ValStart)
		brlo	CALC_PWM_OUT_JMP					; only do something when the value is high enough
		;cpi		OffCounter, 0
		;breq	CALC_PWM_MOT_OFF_START
		;dec		OffCounter
		;rjmp	CALC_PWM_OUT
  CALC_PWM_MOT_OFF_START:
		ori		Status, 1<<ONSTATE						; set the state to on when the value is high enough
		;ldi		OffCounter, TimeOff						; reset the off counter
		rjmp	CALC_PWM_MOT_ON
  CALC_PWM_OUT_JMP:
		rjmp	CALC_PWM_OUT
  ;CALC_PWM_OFF_OUT_JMP:
		;ldi		OffCounter, TimeOff						; reset the off counter
		;rjmp	CALC_PWM_OUT
 CALC_PWM_MOT_ON:
		lds		Temp2, AnaResH
		lds		Temp, AnaResL
		subi	Temp, LOW(ValStop)
		sbci	Temp2, HIGH(ValStop)					; test if the input is larger than the stop value
		brsh	CALC_PWM_MOT_ON_RUN
		cpi		StopCounter, TimeStop
		brsh	CALC_PWM_MOT_ON_STOP					; stop the fan after the stop time
		inc		StopCounter
		ldi		Temp, PWMMin
		rcall	TMR0_SET_DUTY
		rjmp	CALC_PWM_OUT
 CALC_PWM_MOT_ON_STOP:
		clr		StopCounter;
		andi	Status, ~(1<<ONSTATE)					; value is smaller, stop the motor
		clr		Temp
		rcall	TMR0_SET_DUTY							; output 0 percent duty ratio
		rjmp	CALC_PWM_OUT
 CALC_PWM_MOT_ON_RUN:
		clr		StopCounter								; clear the stop counter
		sbrc	Status, HIGHMODE
		rjmp	CALC_PWM_MOT_ON_HIGH
		sbrs	Status, PROGMODE						; test if in on/off mode
		rjmp	CALC_PWM_MOT_ON_LIN
		ldi		Temp2, MaxDuty
		rcall	EEP_READ
		rcall	TMR0_SET_DUTY
		rjmp	CALC_PWM_OUT
 CALC_PWM_MOT_ON_HIGH:
		ldi		Temp2, HighDuty
		rcall	EEP_READ
		rcall	TMR0_SET_DUTY
		rjmp	CALC_PWM_OUT
 CALC_PWM_MOT_ON_LIN:									; in linear mode
		cpi		StartCounter, 0
		breq	CALC_PWM_MOT_ON_LIN_RUN					; change to running mode after startup
		dec		StartCounter							; count the start time
		ldi		Temp, PWMStart
		rcall	TMR0_SET_DUTY							; output the startup duty ratio
		rjmp	CALC_PWM_OUT
 CALC_PWM_MOT_ON_LIN_RUN:
		cpi		Temp2, 1
		brsh	CALC_PWM_CUT_MAX
		mov		Temp3, Temp								; backup Temp
		ldi		Temp2, LinCoeff							; load the multiplier
		rcall	EEP_READ
		mov		Temp2, Temp								; prepare the multiplication
		mov		Temp, Temp3
		rcall	MUL_8_6
		cp		Temp6, One
		brsh	CALC_PWM_CUT_MAX						; limit PWM at range overflow
		mov		Temp, Temp5								; move the result
		ldi		Temp3, PWMMin
		add		Temp, Temp3								; add the minimum duty ratio
		brcs	CALC_PWM_CUT_MAX						; cut to max. duty ratio if overflow occurred
		mov		Temp3, Temp
		ldi		Temp2, MaxDuty
		rcall	EEP_READ								; read the max. duty ratio from EEPROM
		cp		Temp, Temp3
		brlo	CALC_PWM_CUT_MAX
		mov		Temp, Temp3
		rcall	TMR0_SET_DUTY							; output the duty ratio
		rjmp	CALC_PWM_OUT
 CALC_PWM_CUT_MAX:										; output the max. duty ratio
		ldi		Temp2, MaxDuty
		rcall	EEP_READ								; read the max. duty ratio from EEPROM
		rcall	TMR0_SET_DUTY
 CALC_PWM_OUT:
		ret

MUL_8_6:												; multiply temp(8bit) by temp2(6bit)
															; result is stored in temp6:temp5
		ldi		Temp3, 6									; number of bits to be multiplied
		mov		Temp5, Null									; result low byte
		mov		Temp6, Null									; result high byte
		clr		Temp4										; additional register to shift temp
  MUL_8_6_I:
		lsr		Temp2
		brcc	MUL_8_6_NUL
		add		Temp5, Temp
		adc		Temp6, Temp4
   MUL_8_6_NUL:
		lsl		Temp
		rol		Temp4
		dec		Temp3
		brne	MUL_8_6_I
															; finally divide by 32
		ldi		Temp3, 5
  MUL_8_6_DIV:
		lsr		Temp6
		ror		Temp5
		dec		Temp3
		brne	MUL_8_6_DIV
		ret